@@ -20,14 +20,13 @@ import uuid
 
 from concurrent import futures
 from oslo_log import log as logging
-from oslo_serialization import jsonutils
 from oslo_utils import importutils
 from oslo_utils import timeutils
 from pika import exceptions as pika_exceptions
 from pika import spec as pika_spec
 import pika_pool
-import retrying
 import six
+import tenacity
 
 
 import oslo_messaging
@@ -101,7 +100,6 @@ class PikaIncomingMessage(base.IncomingMessage):
         self._version = version
 
         self._content_type = properties.content_type
-        self._content_encoding = properties.content_encoding
         self.unique_id = properties.message_id
 
         self.expiration_time = (
@@ -109,15 +107,16 @@ class PikaIncomingMessage(base.IncomingMessage):
             time.time() + float(properties.expiration) / 1000
         )
 
-        if self._content_type != "application/json":
+        try:
+            serializer = pika_drv_cmns.MESSAGE_SERIALIZERS[self._content_type]
+        except KeyError:
             raise NotImplementedError(
-                "Content-type['{}'] is not valid, "
-                "'application/json' only is supported.".format(
+                "Content-type['{}'] is not supported.".format(
                     self._content_type
                 )
             )
 
-        message_dict = jsonutils.loads(body, encoding=self._content_encoding)
+        message_dict = serializer.load_from_bytes(body)
 
         context_dict = {}
 
@@ -181,7 +180,7 @@ class RpcPikaIncomingMessage(PikaIncomingMessage, base.RpcIncomingMessage):
         :param failure: Tuple, should be a sys.exc_info() tuple.
             Should be None if RPC request was successfully processed.
 
-        :return RpcReplyPikaIncomingMessage, message with reply
+        :return RpcReplyPikaIncomingMessage: message with reply
         """
 
         if self.reply_q is None:
@@ -190,7 +189,6 @@ class RpcPikaIncomingMessage(PikaIncomingMessage, base.RpcIncomingMessage):
         reply_outgoing_message = RpcReplyPikaOutgoingMessage(
             self._pika_engine, self.msg_id, reply=reply, failure_info=failure,
             content_type=self._content_type,
-            content_encoding=self._content_encoding
         )
 
         def on_exception(ex):
@@ -203,14 +201,22 @@ class RpcPikaIncomingMessage(PikaIncomingMessage, base.RpcIncomingMessage):
             else:
                 return False
 
-        retrier = retrying.retry(
-            stop_max_attempt_number=(
-                None if self._pika_engine.rpc_reply_retry_attempts == -1
-                else self._pika_engine.rpc_reply_retry_attempts
-            ),
-            retry_on_exception=on_exception,
-            wait_fixed=self._pika_engine.rpc_reply_retry_delay * 1000,
-        ) if self._pika_engine.rpc_reply_retry_attempts else None
+        if self._pika_engine.rpc_reply_retry_attempts:
+            retrier = tenacity.retry(
+                stop=(
+                    tenacity.stop_never
+                    if self._pika_engine.rpc_reply_retry_attempts == -1 else
+                    tenacity.stop_after_attempt(
+                        self._pika_engine.rpc_reply_retry_attempts
+                    )
+                ),
+                retry=tenacity.retry_if_exception(on_exception),
+                wait=tenacity.wait_fixed(
+                    self._pika_engine.rpc_reply_retry_delay
+                )
+            )
+        else:
+            retrier = None
 
         try:
             timeout = (None if self.expiration_time is None else
@@ -297,8 +303,7 @@ class PikaOutgoingMessage(object):
     and send it
     """
 
-    def __init__(self, pika_engine, message, context,
-                 content_type="application/json", content_encoding="utf-8"):
+    def __init__(self, pika_engine, message, context, content_type=None):
         """Parse RabbitMQ message
 
         :param pika_engine: PikaEngine, shared object with configuration and
@@ -306,19 +311,23 @@ class PikaOutgoingMessage(object):
         :param message: Dictionary, user's message fields
         :param context: Dictionary, request context's fields
         :param content_type: String, content-type header, defines serialization
-            mechanism
-        :param content_encoding: String, defines encoding for text data
+            mechanism, if None default content-type from pika_engine is used
         """
 
         self._pika_engine = pika_engine
 
-        self._content_type = content_type
-        self._content_encoding = content_encoding
+        self._content_type = (
+            content_type if content_type is not None else
+            self._pika_engine.default_content_type
+        )
 
-        if self._content_type != "application/json":
+        try:
+            self._serializer = pika_drv_cmns.MESSAGE_SERIALIZERS[
+                self._content_type
+            ]
+        except KeyError:
             raise NotImplementedError(
-                "Content-type['{}'] is not valid, "
-                "'application/json' only is supported.".format(
+                "Content-type['{}'] is not supported.".format(
                     self._content_type
                 )
             )
@@ -335,12 +344,11 @@ class PikaOutgoingMessage(object):
         msg = self.message.copy()
 
         if self.context:
-            for key, value in six.iteritems(self.context):
+            for key, value in self.context.items():
                 key = six.text_type(key)
                 msg['_$_' + key] = value
 
         props = pika_spec.BasicProperties(
-            content_encoding=self._content_encoding,
             content_type=self._content_type,
             headers={_VERSION_HEADER: _VERSION},
             message_id=self.unique_id,
@@ -438,8 +446,8 @@ class PikaOutgoingMessage(object):
             for routing into durable queues
         :param stopwatch: StopWatch, stopwatch object for calculating
             allowed timeouts
-        :param retrier: retrying.Retrier, configured retrier object for sending
-            message, if None no retrying is performed
+        :param retrier: tenacity.Retrying, configured retrier object for
+            sending message, if None no retrying is performed
         """
         msg_props.delivery_mode = 2 if persistent else 1
 
@@ -447,8 +455,7 @@ class PikaOutgoingMessage(object):
                 if confirm else
                 self._pika_engine.connection_without_confirmation_pool)
 
-        body = jsonutils.dump_as_bytes(msg_dict,
-                                       encoding=self._content_encoding)
+        body = self._serializer.dump_as_bytes(msg_dict)
 
         LOG.debug(
             "Sending message:[body:%s; properties: %s] to target: "
@@ -476,8 +483,8 @@ class PikaOutgoingMessage(object):
             for routing into durable queues
         :param stopwatch: StopWatch, stopwatch object for calculating
             allowed timeouts
-        :param retrier: retrying.Retrier, configured retrier object for sending
-            message, if None no retrying is performed
+        :param retrier: tenacity.Retrying, configured retrier object for
+            sending message, if None no retrying is performed
         """
         msg_dict, msg_props = self._prepare_message_to_send()
 
@@ -490,10 +497,9 @@ class RpcPikaOutgoingMessage(PikaOutgoingMessage):
     """PikaOutgoingMessage implementation for RPC messages. It adds
     possibility to wait and receive RPC reply
     """
-    def __init__(self, pika_engine, message, context,
-                 content_type="application/json", content_encoding="utf-8"):
+    def __init__(self, pika_engine, message, context, content_type=None):
         super(RpcPikaOutgoingMessage, self).__init__(
-            pika_engine, message, context, content_type, content_encoding
+            pika_engine, message, context, content_type
         )
         self.msg_id = None
         self.reply_q = None
@@ -508,8 +514,8 @@ class RpcPikaOutgoingMessage(PikaOutgoingMessage):
             reply. If None - return immediately without reply waiting
         :param stopwatch: StopWatch, stopwatch object for calculating
             allowed timeouts
-        :param retrier: retrying.Retrier, configured retrier object for sending
-            message, if None no retrying is performed
+        :param retrier: tenacity.Retrying, configured retrier object for
+            sending message, if None no retrying is performed
         """
         msg_dict, msg_props = self._prepare_message_to_send()
 
@@ -549,7 +555,7 @@ class RpcReplyPikaOutgoingMessage(PikaOutgoingMessage):
     correlation_id AMQP property to link this reply with response
     """
     def __init__(self, pika_engine, msg_id, reply=None, failure_info=None,
-                 content_type="application/json", content_encoding="utf-8"):
+                 content_type=None):
         """Initialize with reply information for sending
 
         :param pika_engine: PikaEngine, shared object with configuration and
@@ -559,8 +565,7 @@ class RpcReplyPikaOutgoingMessage(PikaOutgoingMessage):
         :param failure_info: Tuple, should be a sys.exc_info() tuple.
             Should be None if RPC request was successfully processed.
         :param content_type: String, content-type header, defines serialization
-            mechanism
-        :param content_encoding: String, defines encoding for text data
+            mechanism, if None default content-type from pika_engine is used
         """
         self.msg_id = msg_id
 
@@ -588,7 +593,7 @@ class RpcReplyPikaOutgoingMessage(PikaOutgoingMessage):
             msg = {'s': reply}
 
         super(RpcReplyPikaOutgoingMessage, self).__init__(
-            pika_engine, msg, None, content_type, content_encoding
+            pika_engine, msg, None, content_type
         )
 
     def send(self, reply_q, stopwatch=pika_drv_cmns.INFINITE_STOP_WATCH,
@@ -598,8 +603,8 @@ class RpcReplyPikaOutgoingMessage(PikaOutgoingMessage):
         :param reply_q: String, queue name for sending reply
         :param stopwatch: StopWatch, stopwatch object for calculating
             allowed timeouts
-        :param retrier: retrying.Retrier, configured retrier object for sending
-            message, if None no retrying is performed
+        :param retrier: tenacity.Retrying, configured retrier object for
+            sending message, if None no retrying is performed
         """
 
         msg_dict, msg_props = self._prepare_message_to_send()
